#include "Arduino.h"
#include "ADAU1701_I2C.h"
#include <Wire.h>

ADAU1701_I2C::ADAU1701_I2C(uint8_t I2C_addr)
{
  _I2C_addr = I2C_addr;
}

void ADAU1701_I2C::  Read_block(byte MSB, byte LSB, uint8_t (& Data_block) [16], int count)
{
  size_t count_t = count;
  uint8_t addr[] = {MSB, LSB};
  Wire.beginTransmission(_I2C_addr);   // transmit to device at I2C_addr
  Wire.write(addr, 2);
  Wire.endTransmission(false);           //Address counter set
  Wire.requestFrom(_I2C_addr, count_t, true); //request a block of bytes
  uint8_t x = 0;
  while (Wire.available()) Data_block[x++] = Wire.read();
}

void ADAU1701_I2C::  I2C_write_Mult(uint8_t addr_MSB, uint8_t addr_LSB, uint8_t count, uint8_t command_buffer[])
{
  uint8_t addr[7];
  addr[0] = addr_MSB;
  addr[1] = addr_LSB;
  for (uint8_t x = 0; x < count; x++) {
    addr[x + 2] = command_buffer[x];        //copy data into buffer
  }

  Wire.beginTransmission(_I2C_addr);     // transmit to device at I2C_addr
  Wire.write(addr, count + 2);          //write subaddress followed by data
  Wire.endTransmission();
}

void ADAU1701_I2C::  safeload_addr(word v_reg, byte safeload_reg_num) {    //put address into safeload registers
  uint8_t addr_MSB = 0x08;            //subaddress MSB
  uint8_t addr_LSB;                   //subaddress LSB
  uint8_t count = 2;                  //length = 2 bytes
  uint8_t Cmd_Data[5];                //allow up to 5 bytes of Command data

  addr_LSB = 0x15 + safeload_reg_num;              //LSB of safeload address register
  count = 2;                  //two bytes

  Cmd_Data[1] = lowByte(v_reg);
  Cmd_Data[0] = lowByte(v_reg >> 8);
  I2C_write_Mult(addr_MSB, addr_LSB, count, Cmd_Data);
}


void ADAU1701_I2C::  safeload_data(byte safeload_reg_num, uint8_t safeload_data_array[]) {          //put data into safeload registers
  uint8_t addr_MSB = 0x08;            //subaddress MSB
  uint8_t addr_LSB;                   //subaddress LSB
  uint8_t count;                      //length = 2 bytes
  uint8_t Cmd_Data[5];                //allow up to 5 bytes of Command data

  //Parameter_1_MSB = 0x08;
  addr_LSB = 0x10 + safeload_reg_num;     //address for safeload register #X
  count = 5;
  for (int i = 0; i < count; i++) {
    Cmd_Data[i] = safeload_data_array[i];
  }
  I2C_write_Mult(addr_MSB, addr_LSB, count, Cmd_Data);
}


void ADAU1701_I2C::  biquad_safe_load(word filter_reg, byte filter_coefs[][4] )  {
  uint8_t Cmd_Data[5];                //allow up to 5 bytes of Command data
  //filter_table_pointer is pointing to the right table
  Cmd_Data[0] = 0;                                        //extra bits needed for 40-bit data; this won't change
  for (int j = 0; j < 5; j++) {
    safeload_addr(filter_reg + j, j);                     //put addresses for coefficient in safeload addresses
    for (int i = 0; i < 4; i++) {
      Cmd_Data[i + 1] = filter_coefs[j][i];
      safeload_data(j, Cmd_Data);                                     //put data for coefficients in safeload data registers
    }                                                       //repeat for all 5 biquad coefficients
    safeload_xfer();                                        //set the transfer bit (update all five coefficients)
  }
}

void ADAU1701_I2C::  safeload_xfer() {          //toggle safeload transfer bit
  uint8_t addr_MSB = 0x08;            //subaddress MSB
  uint8_t addr_LSB;                   //subaddress LSB
  uint8_t count = 2;                  //length = 2 bytes
  uint8_t Cmd_Data[5];                //allow up to 5 bytes of Command data

  addr_LSB = 0x1C;           //address for safeload register #0
  count = 2;              //length = 2 bytes
  Cmd_Data[0] = 0;
  if (setup_done == false)       //keep audio off if muted or during setup
  {
    Cmd_Data[1] = 0x34; //transfer bit is D5
  }
  else
  {
    Cmd_Data[1] = 0x3C; //transfer bit is D5
  }
  I2C_write_Mult(addr_MSB, addr_LSB, count, Cmd_Data);   //set safeload transfer bit+++
}


void ADAU1701_I2C::  ADAU1701_soft_reset() {
  //reset  Address 081C; DSP Core Control
  uint8_t addr_MSB = 0x08;            //subaddress MSB
  uint8_t addr_LSB;                   //subaddress LSB
  uint8_t count = 2;                  //length = 2 bytes
  uint8_t Cmd_Data[5];                //allow up to 5 bytes of Command data

  addr_LSB = 0x1C;   //subaddress LSB = DSP Core Control
  count = 2;      //length
  Cmd_Data[0] = 0;      //Data MSB = 0
  Cmd_Data[1] = 0;      //Data LSB = 0 = stop everything
  I2C_write_Mult(addr_MSB, addr_LSB, count, Cmd_Data);   //send the command
  delay(100);           //wait for at least two cycles
}


void ADAU1701_I2C::  Load_program(const byte* Prog_table_pointer) {
  /*Program data uses ADAU1701 EEPROM Message format:
    Message ID    Message Type    Following Bytes
    0x00          End             Note:  this message not used
    0x01          Write           Two bytes for message length + data
    0x02          Delay           Two bytes for delay (not used)
    0x03          No operation executed
    0x04          Set multiple writeback (not used)
    0x05          Set WB to falling edge sensitive (not used)
    0x06          End and wait for writeback
    Note:  SigmaStudio generates two Program Memory outputs.  One is formatted for the EEPROM, using ADI's message format
    and the other is just the code formatted as HEX bytes.  The latter would be easier to use, but the code right now uses
    the EEPROM format.  That's because the HEX byte format wasn't available in early versions of SigmaStudio when this routine
    was first designed.  So this routine should probably be updated to use the HEX format.
  */

  long table_counter = 0;
  byte  table_data = 0;
  byte  data_available = 1;
  byte instruction_length = 5;
  const byte* table_pointer;

  while (data_available > 0) {

    table_data = pgm_read_byte_far(Prog_table_pointer + table_counter);

    switch (table_data) {
      case 0x00:
        data_available = 0;   //END
        break;
      case 0x01:
        table_counter++;
        table_pointer = Prog_table_pointer;
        instruction_length = 5;
        ADAU1701_write_table(table_pointer, instruction_length);   //send the program data to the ADAU1701
        data_available = 0;   //END
        break;
      case 0x03:
        table_counter++;
        break;
      case 0x06:
        data_available = 0;   //END
        break;
      default:
        data_available = 0;   //default is stop and exit
        break;
    }
  }
}

void ADAU1701_I2C::  Load_parameters(const byte* Param_table_pointer) {
  long table_counter = 0;
  byte  table_data = 0;
  byte  data_available = 1;
  byte instruction_length = 4;
  const byte* table_pointer;

  while (data_available > 0) {

    table_data = pgm_read_byte_far(Param_table_pointer + table_counter);

    switch (table_data) {
      case 0x00:
        data_available = 0;   //END
        break;
      case 0x01:
        table_counter++;
        table_pointer = Param_table_pointer;
        instruction_length = 4;
        ADAU1701_write_table(table_pointer, instruction_length);   //send the parameter data to the ADAU1701
        data_available = 0;   //END
        break;
      case 0x03:
        table_counter++;
        break;
      case 0x06:
        data_available = 0;   //END
        break;
      default:
        data_available = 0;   //default is stop and exit
        break;
    }
  }
}

void ADAU1701_I2C::  ADAU1701_write_table( const byte* t_base, byte inst_len) {
  long t_count = 1;
  long table_length = 0;
  byte  start_addr_MSB;
  byte  start_addr_LSB;
  word Temp_int;
  uint8_t addr_MSB = 0x08;            //subaddress MSB
  uint8_t addr_LSB = 0x27;            //subaddress LSB = DAC Setup
  uint8_t count = 2;                  //length = 2 bytes
  uint8_t Cmd_Data[6];                //allow up to 6 bytes of Command data


  table_length = pgm_read_byte_far(t_base + t_count) << 8; //get length in bytes
  t_count++;
  table_length =  table_length + pgm_read_byte_far(t_base + t_count) - 2;

  t_count = t_count + 2;                                //skip over one byte
  start_addr_MSB = pgm_read_byte_far(t_base + t_count);
  t_count++;
  start_addr_LSB = pgm_read_byte_far(t_base + t_count);
  t_count++;                                                  //point to start of program instruction


  while (t_count <= table_length) {
    for (int i = 0; i < inst_len; i++)
    {
      Cmd_Data[i] = pgm_read_byte_far(t_base + t_count);
      t_count++;                //load up the command buffer & point to next instruction
    }
    addr_MSB = start_addr_MSB;
    addr_LSB = start_addr_LSB;
    count = inst_len;

    I2C_write_Mult(addr_MSB, addr_LSB, count, Cmd_Data);

    Temp_int = start_addr_MSB * 256 + start_addr_LSB;
    Temp_int++;                           //increment the instruction counter
    start_addr_LSB = lowByte(Temp_int);
    start_addr_MSB = lowByte(Temp_int >> 8);  //save the instruction address as bytes

  }
}

void ADAU1701_I2C::  Load_registers() {
  uint8_t addr_MSB = 0x08;            //subaddress MSB
  uint8_t addr_LSB = 0x27;            //subaddress LSB = DAC Setup
  uint8_t count = 2;                  //length = 2 bytes
  uint8_t Cmd_Data[5];                //allow up to 5 bytes of Command data

  //DAC setup.  Address 0827; initialize DACs = 01
  Cmd_Data[0] = 0;        //Data MSB = 0
  Cmd_Data[1] = 1;        //Data LSB = 1 = Initialize DAC's
  I2C_write_Mult(addr_MSB, addr_LSB, count, Cmd_Data);        //Send the command


  //Serial Output Control Register.  Address 081E
  addr_LSB = 0x1E;        //subaddress LSB (MSB unchanged)
  count = 2;              //length = 2 bytes
  Cmd_Data[0] = 0x8;      //Master, with internal clock divider = 16.  BCLK = 3.072MHz
  Cmd_Data[1] = 0x0;      //default to stereo 24-bit
  //Cmd_Data[0] = 0x0D;      //Master, with internal clock divider = 4.  BCLK = 12.288 Mhz  LRCLK = 192KHz
  //Cmd_Data[1] = 0x20;      //turn on TDM for testing the 4-channel amp board
  I2C_write_Mult(addr_MSB, addr_LSB, count, Cmd_Data);     //Send the command

  //Multipurpose Pin Configuration Register #1.  Address 0820
  addr_LSB = 0x20;        //subaddress LSB (MSB unchanged)
  count = 3;              //length
  Cmd_Data[0] = 0x44;     //MP5, MP4 = Serial data port
  Cmd_Data[1] = 0x44;     //MP3, MP2 = Serial data port
  Cmd_Data[2] = 0x44;     //MP1, MP0 = Serial data port
  I2C_write_Mult(addr_MSB, addr_LSB, count, Cmd_Data);     //Send the command

  //Multipurpose Pin Configuration Register #2.  Address 0821
  addr_LSB = 0x21;        //subaddress LSB (MSB unchanged)
  count = 3;              //length
  Cmd_Data[0] = 0x44;     //MP11, MP10 = Serial data port
  Cmd_Data[1] = 0x44;     //MP9, MP8 = Serial data
  Cmd_Data[2] = 0x44;     //MP7 =  MP6 = Serial data
  I2C_write_Mult(addr_MSB, addr_LSB, count, Cmd_Data);     //Send the command
}
